const BYTE_LENGTH: usize = 20;

pub type Bytes = [u8; BYTE_LENGTH];

#[derive(Clone, Copy, PartialEq, PartialOrd, Ord, Hash, Eq)]
pub struct SUID(Bytes);

impl SUID {
    /// Generates an SUID using SystemTime and a random string.
    /// The total length of the byte array will be 20.
    pub fn new() -> Self {
        use rand::distributions::{Alphanumeric, DistString};

        let time = std::time::SystemTime::now()
            .duration_since(std::time::UNIX_EPOCH)
            .unwrap()
            .as_micros();
        let mut bytes: Bytes = [0; BYTE_LENGTH];
        let b_string = format!(
            "{}{}",
            time,
            Alphanumeric.sample_string(&mut rand::thread_rng(), 4)
        );
        let b = b_string.as_bytes();

        if b.len() != BYTE_LENGTH {
            tracing::error!("Byte length wrong. Length = {}", b.len());
        }

        bytes.copy_from_slice(b);
        Self(bytes)
    }
    /// Parses an SUID as hex.
    /// Use as_string instead.
    pub fn parse(self) -> String {
        format!("{:02x?}", self.0)
    }

    /// Returns an SUID as a hexadecimal string.
    pub fn as_string(self) -> String {
        let mut string = "".to_string();

        for v in self.0 {
            string = format!("{}{:02x?}", string, v);
        }

        return string;
    }
}
